/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.web.charts;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PiePlot;
import org.jfree.chart.title.TextTitle;
import org.jfree.data.general.DefaultPieDataset;
import org.jfree.ui.RectangleInsets;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.Map;
import java.util.StringTokenizer;

public class PieChart extends BaseChartWeb implements Chart {

  private DefaultPieDataset dataset;

  public PieChart(Map<String, String> params) {
    super(params);
    jfreechart = new JFreeChart(null, TextTitle.DEFAULT_FONT, new PiePlot(), false);
  }

  protected BufferedImage getChartImage() throws IOException {
    configure();
    return getBufferedImage(jfreechart);
  }

  private void configure() {
    configureChart(jfreechart, false);
    configureDataset();
    configurePlot();
    applyParams();
  }

  private void configureDataset() {
    dataset = new DefaultPieDataset();
  }

  private void configurePlot() {
    PiePlot plot = (PiePlot) jfreechart.getPlot();
    plot.setNoDataMessage(DEFAULT_MESSAGE_NODATA);
    plot.setInsets(RectangleInsets.ZERO_INSETS);
    plot.setDataset(dataset);
    plot.setBackgroundAlpha(0.0f);
    plot.setCircular(true);
    plot.setLabelGenerator(null);
    plot.setIgnoreNullValues(true);
    plot.setIgnoreZeroValues(true);
    plot.setShadowPaint(null);
    plot.setLabelLinkMargin(0.0);
    plot.setInteriorGap(0.02);
    plot.setMaximumLabelWidth(0.10);
  }

  private void configureColors(String colors) {
    try {
      if (colors != null && colors.length() > 0) {
        StringTokenizer stringTokenizer = new StringTokenizer(colors, ",");
        int i = 0;
        while (stringTokenizer.hasMoreTokens()) {
          ((PiePlot) jfreechart.getPlot()).setSectionPaint(Integer.toString(i), Color.decode("0x" + stringTokenizer.nextToken()));
          i++;
        }
      } else {
        configureDefaultColors();
      }
    }
    catch (Exception e) {
      configureDefaultColors();
    }
  }

  private void configureDefaultColors() {
    PiePlot plot = (PiePlot) jfreechart.getPlot();
    plot.setSectionPaint("0", SERIE1_COLOR);
    plot.setSectionPaint("1", SERIE2_COLOR);
    plot.setSectionPaint("2", SERIE3_COLOR);
    plot.setSectionPaint("3", SERIE4_COLOR);
    plot.setSectionPaint("4", SERIE5_COLOR);
  }

  private void applyParams() {
    applyCommonParams();

    configureColors(params.get(BaseChartWeb.CHART_PARAM_COLORS));
    addMeasures(params.get(BaseChartWeb.CHART_PARAM_VALUES));

    // -- Plot
    PiePlot plot = (PiePlot) jfreechart.getPlot();
    plot.setOutlineVisible(isParamValueValid(params.get(BaseChartWeb.CHART_PARAM_OUTLINE_VISIBLE)) && Boolean.getBoolean(params.get(BaseChartWeb.CHART_PARAM_OUTLINE_VISIBLE)));
  }

  private void addMeasures(String values) {
    if (values != null && values.length() > 0) {
      StringTokenizer st = new StringTokenizer(values, ",");
      int i = 0;
      while (st.hasMoreTokens()) {
        double measure = 0;
        try {
          measure = Double.parseDouble(st.nextToken());
        }
        catch (NumberFormatException e) {
        }
        dataset.setValue(Integer.toString(i), measure);
        i++;
      }
    }
  }

}